// Copyright (C) 2012 MicroStrain, Inc.

/*********************
*	node.cpp
*	node.cpp holds the implementation for each individual
*	function that applies to the nodes.  For a better understanding
*	of each function, please, reference the desired function.
*********************/

#include "2400_sdk.h"
#include "stdafx.h"

#include "node.h"

/*	used for serial (comm) port functionality	*/
#include "serial.h"
/*	used for console output functionality	*/
#include "console_commands.h"
/*	create a c-style array at runtime that cleans itself up.	*/
#include "dynamic_buffer.h"
/*	used to get information on the sample rate value read in	*/
#include "sampling_rates.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CMapPosToCString strCmd_info;
/*	crCmd_color is the list of colors for outputting to the console window.  When information is
*	output to the screen, the drawing function looks up the corresponding color based on its
*	position in the console window and sets the text color to that index's color.	*/
extern CMapPosToColor crCmd_color;

/***********************
*	nd_ping
*	nd_ping attempts to ping the node address, usNode, over the port, hPort.
*	Command Format
*		Length:		3 bytes
*		Byte 1:		0x02
*		Byte 2:		MSB of the node's address
*		Byte 3:		LSB of the node's address
*	Good Response:
*		Length:		1 byte
*		Byte 1:		0x02
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to ping
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_ping(/* in */ HANDLE hPort,
			/* in */ unsigned short usNode)
{
	TRACE("nd_ping()\n");
	unsigned char	ucCmd[3] =	{	0x02,	//	ping command byte
									usNode >> 8,	//	MSB of the node address
									usNode & 0xFF	//	LSB of the node address
								};
	unsigned char ucResponse;		//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 3, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to ping node %i\r\nCommand Byte: %#04x\tMSB Node Address: %#04x\tLSB Node Address: %#04x", usNode, ucCmd[0], ucCmd[1], ucCmd[2]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	the only legitimate response is a 0x02	*/
	if (ucResponse != 0x02) {
		/*	append to the command description	*/
		strFormat.Format("Failed to ping node %i\r\nResponse: %#04x", usNode, ucResponse);
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("Ping attempt successful for node %i\r\nResponse: %#04x", usNode, ucResponse);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_ping_long
*	nd_ping_long attempts to ping the node address, usNode, over the port, hPort.
*	Command Format
*		Length:		10 bytes
*		Byte 1:		0xAA		SOP
*		Byte 2:		0x05		Message Type
*		Byte 3:		0x00		Address Mode
*		Byte 4:		MSB of the node's address
*		Byte 5:		LSB of the node's address
*		Byte 6:		0x02
*		Byte 7:		0x00
*		Byte 8:		0x02
*		Byte 9:		MSB of the checksum
*		Byte 10:	LSB of the checksum
*	Good Response:
*		Length:		13 Bytes
*		Byte 1:		0xAA		Valid packet response from base station
*		Byte 2:		0xAA		SOP
*		Byte 3:		0x07		Message Type
*		Byte 4:		0x02		Address Mode
*		Byte 5:		MSB of the node's address
*		Byte 6:		LSB of the node's address
*		Byte 7:		0x02
*		Byte 8:		0x00
*		Byte 9:		0x00
*		Byte 10:	0xXX		LQI
*		Byte 11:	0xXX		RSSI
*		Byte 12:	MSB of the checksum
*		Byte 13:	LSB of the checksum
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to ping
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_ping_long(/* in */ HANDLE hPort,
					/* in */ unsigned short usNode)
{
	TRACE("nd_ping_long()\n");
	unsigned char ucCmd[10] = {	0xAA,			//	SOP
								0x05,			//	Message Type
								0x00,			//	Address Mode
								usNode >> 8,	//	MSB of the node's address
								usNode & 0xFF,	//	LSB of the node's address
								0x02,			//	packet length
								0x00,			
								0x02,
								(0x09 + (usNode >> 8) + (usNode & 0xFF)) >> 8,		//	MSB of the checksum
								(0x09 + (usNode >> 8) + (usNode & 0xFF)) & 0xFF		//	LSB of the checksum
	};

	unsigned char ucResponse[13];		//	holds the response
	DWORD	dwBytes_written = 0,		//	number of bytes written to the port
			dwBytes_read = 0;			//	number of bytes read from the port
	int iResult,						//	return value of port read/write functions
		iPos;							//	position in the console that the command was written to
	CString strFormat;					//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 10, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to ping node %i\r\nSOP: %#04x\t\t\tMessage Type: %#04x\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tMSB of Command Byte: %#04x\tLSB of Command Byte: %#04x\r\nMSB of Checksum: %#04x\t\tLSB of Checksum: %#04x",
			usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3], ucCmd[4], ucCmd[5], ucCmd[6], ucCmd[7],
			ucCmd[8], ucCmd[9]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, ucResponse, 13, dwBytes_read, iPos)) != 0) {
		if (iResult == ERROR_MISMATCHING_BYTE_LENGTH) {
			strFormat.Format("Failed to ping node %i", usNode);
			CString strTemp;
			for (DWORD i = 0; i < dwBytes_read; i++) {
				if (i % 3 == 0)
					strFormat += "\r\n";
				strTemp.Format("Response Byte %i: %#04x\t\t", i, ucResponse[i]);
				strFormat += strTemp;
			}
			strCmd_info[iPos] = strFormat;
		}
		return iResult;
	}

	/*	make sure the first two bytes are 0xAA and the packet is 13 bytes long, otherwise, the packet is bad.	*/
	if (dwBytes_read != 13 || ucResponse[0] != 0xAA || ucResponse[1] != 0xAA) {
		/*	append to the command description	*/
		strFormat.Format("Failed to ping node %i", usNode);
		CString strTemp;
		for (DWORD i = 0; i < dwBytes_read; i++) {
			if (i % 3 == 0)
				strFormat += "\r\n";
			strTemp.Format("Response Byte %i: %#04x\t\t", i, ucResponse[i]);
			strFormat += strTemp;
		}
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("Ping attempt successful for node %i\r\nBase Station Response: %#04x\tSOP: %#04x\t\t\tMessage Type: %#04x\r\nAddress Mode: %#04x\t\tMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tMSB of Command Byte: %#04x\tLSB of Command Byte: %#04x\r\nLQI: %#04x\t\t\tRSSI: %#04x\r\nMSB of Checksum: %#04x\t\tLSB of Checksum: %#04x", usNode,
		ucResponse[0], ucResponse[1], ucResponse[2], ucResponse[3], ucResponse[4], ucResponse[5], ucResponse[6], ucResponse[7], ucResponse[8], ucResponse[9], ucResponse[10], ucResponse[11], ucResponse[12]);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_eeprom_read
*	nd_eeprom_read attempts to read an eeprom location, usLocation, from the node address, usNode,
*	over the port given by handle, hPort
*	Command Format (12 bytes)
*		Byte 1:		0xAA (Start of Packet)
*		Byte 2:		0x05 (Delivery Stop Flag)
*		Byte 3:		0x00 (App Data Type)
*		Byte 4:		0xXX (MSB of the node's address)
*		Byte 5:		0xXX (LSB of the node's address)
*		Byte 6:		0x04 (Payload length)
*		Byte 7:		0x00 (MSB of the command ID)
*		Byte 8:		0x03 (LSB of the command ID)
*		Byte 9:		0xXX (MSB of the location to read)
*		Byte 10:	0xXX (LSB of the location to read)
*		Byte 11:	0xXX (MSB of the checksum of bytes 2-10)
*		Byte 12:	0xXX (LSB of the checksum of bytes 2-10)
*
*	Good Response (12 bytes + 1 byte from base):
*					0x01 - 1 byte response directly from the base station to verify that the command was correctly reveived and sent to the Node
*		
*		Byte 1:		0xAA (Start of Packet)
*		Byte 2:		0x00 (Delivery Stop Flag)
*		Byte 3:		0x00 (App Data Type)
*		Byte 4:		0xXX (MSB of the node's address)
*		Byte 5:		0xXX (LSB of the node's address)
*		Byte 6:		0x02 (Payload length)
*		Byte 7:		0xXX (MSB of the EEprom value read)
*		Byte 8:		0xXX (LSB of the EEprom value read)
*		Byte 9:		0xXX (LQI - Junk, Throw Out)
*		Byte 10:	0xXX (Base Station RSSI)
*		Byte 11:	0xXX (MSB of the checksum of bytes 2-8)
*		Byte 12:	0xXX (LSB of the checksum of bytes 2-8)

*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to read
*			usLocation is an unsigned short holding the location to read
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_eeprom_read(/* in */ HANDLE hPort,
					/* in */ unsigned short usNode,
					/* in */ unsigned short usLocation,
					/* out */ unsigned short & usResponse)
{
	unsigned short nodeMSB = usNode >> 8;
	unsigned short nodeLSB = usNode & 0xFF;
	unsigned short locationMSB = usLocation >> 8;
	unsigned short locationLSB = usLocation & 0xFF;

	TRACE("nd_eeprom_read()\n");
	unsigned char	ucCmd[12] =	{	0xAA,																//	Start of Packet
									0x05,																//	Delivery Stop Flag
									0x00,																//	App Data Type
									nodeMSB,															//	MSB of the node address
									nodeLSB,															//	LSB of the node address
									0x04,																//	Payload Length
									0x00,																//	MSB of the command ID
									0x03,																//	LSB of the command ID
									locationMSB,														//	MSB of the location to read
									locationLSB,														//	LSB of the location to read
									(0x0C + nodeMSB + nodeLSB + locationMSB + locationLSB) >> 8,		//	MSB of the checksum (bytes 2 - 10)
									(0x0C + nodeMSB + nodeLSB + locationMSB + locationLSB) & 0xFF		//	LSB of the checksum (bytes 2 - 10)
								};

	unsigned char ucResponse[13];	//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 12, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to read EEPROM location %i on node %i\r\nCommand Byte: 0x03\tMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nMSB EEPROM Location: %#04x\tLSB EEPROM Location: %#04x",
		usLocation, usNode, ucCmd[3], ucCmd[4], ucCmd[8], ucCmd[9]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, ucResponse, 13, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	calculate the checksum */
	unsigned short calculatedChecksum = 0;

	//checksum is bytes 2 through 8 (skip over the 0xAA from the base station and the Start of Packet byte)
	for(int i = 2; i <= 8; i++)
	{
		//add each byte to the calculated checksum
		calculatedChecksum += ucResponse[i];
	}

	if (calculatedChecksum != ((ucResponse[11] << 8) + ucResponse[12]))
	{
		/*	append to the command description	*/
		strFormat.Format("A bad checksum was calculated on the read from node %i\r\nReturned Checksum: %#04x\r\nCalculated Checksum: %#04x", usNode, (ucResponse[11] << 8) + ucResponse[12], calculatedChecksum);
		strCmd_info[iPos] = strFormat;
		return ERROR_BAD_CHECKSUM;
	}

	/*	store the response	*/
	usResponse = (ucResponse[7] << 8) + ucResponse[8];

	/*	append to the command description	*/
	strFormat.Format("Successful read attempt on EEPROM location %i on node %i\r\nRead Value: %i\r\nChecksum: %#04x", 
		usLocation, usNode, usResponse, (ucResponse[11] << 8) + ucResponse[12]);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_eeprom_write
*	nd_eeprom_write attempts to write a value, usValue, to an EEPROM location, usLocation, on a node,
*	usNode, over the port opened by hPort.
*	Command Format (14 bytes)
*		Byte 1:		0xAA (Start of Packet)
*		Byte 2:		0x05 (Delivery Stop Flag)
*		Byte 3:		0x00 (App Data Type)
*		Byte 4:		0xXX (MSB of node's address)
*		Byte 5:		0xXX (LSB of node's address)
*		Byte 6:		0x06 (Payload Length)
*		Byte 7:		0x00 (MSB of command ID)
*		Byte 8:		0x04 (LSB of command ID)
*		Byte 9:		0xXX (MSB of eeprom address)
*		Byte 10:	0xXX (LSB of eeprom address)
*		Byte 11:	0xXX (MSB of value to write)
*		Byte 12:	0xXX (LSB of value to write)
*		Byte 13:	0xXX (MSB of checksum of bytes 2-12)
*		Byte 14:	0xXX (LSB of checksum of bytes 2-12)
*
*	Good Response (12 bytes + 1 byte from base):
*					0x01 - 1 byte response directly from the base station to verify that the command was correctly reveived and sent to the Node
*		
*		Byte 1:		0xAA (Start of Packet)
*		Byte 2:		0x00 (Delivery Stop Flag)
*		Byte 3:		0x00 (App Data Type)
*		Byte 4:		0xXX (MSB of the node's address)
*		Byte 5:		0xXX (LSB of the node's address)
*		Byte 6:		0x02 (Payload length)
*		Byte 7:		0x00 (MSB of the command ID)
*		Byte 8:		0x04 (LSB of the command ID)
*		Byte 9:		0xXX (LQI - Junk, Throw Out)
*		Byte 10:	0xXX (Base Station RSSI)
*		Byte 11:	0xXX (MSB of the checksum of bytes 2-8)
*		Byte 12:	0xXX (LSB of the checksum of bytes 2-8)
*
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to write
*			usLocation is an unsigned short holding the location to write
*			usValue is an unsigned short holding the value to write
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_eeprom_write(/* in */ HANDLE hPort,
					/* in */ unsigned short usNode,
					/* in */ unsigned short usLocation,
					/* in */ unsigned short usValue)
{
	unsigned short nodeMSB = usNode >> 8;
	unsigned short nodeLSB = usNode & 0xFF;
	unsigned short locationMSB = usLocation >> 8;
	unsigned short locationLSB = usLocation & 0xFF;
	unsigned short valueMSB = usValue >> 8;
	unsigned short valueLSB = usValue & 0xFF;

	TRACE("nd_eeprom_write()\n");
	unsigned char	ucCmd[14] =	{	0xAA,				//	Start of Packet
									0x05,				//	Delivery Stop Flag
									0x00,				//	App Data Type
									nodeMSB,			//	MSB of the node address
									nodeLSB,			//	LSB of the node address
									0x06,				//	Payload Length
									0x00,				//	MSB of the Command ID
									0x04,				//	LSB of the Command ID
									locationMSB,		//	MSB of location to write
									locationLSB,		//	LSB of location to write
									valueMSB,			//	MSB of value to write
									valueLSB,			//	LSB of value to write
									(0x0F + nodeMSB + nodeLSB + locationMSB + locationLSB + valueMSB + valueLSB) >> 8,		//	MSB of the checksum (bytes 2 - 12)
									(0x0F + nodeMSB + nodeLSB + locationMSB + locationLSB + valueMSB + valueLSB) & 0xFF		//	LSB of the checksum (bytes 2 - 12)
								};

	unsigned char ucResponse[13];	//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 14, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to write %i to EEPROM location %i on node %i\r\nCommand Byte: 0x04\tMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nEEPROM Location: %#04x\r\nMSB Write Value: %#04x\tLSB Write Value: %#04x\r\nMSB Checksum: %#04x\tLSB Checksum: %#04x", 
		usValue, usLocation, usNode, ucCmd[3], ucCmd[4], ucCmd[8], ucCmd[9], ucCmd[10], ucCmd[11], ucCmd[12], ucCmd[13]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, ucResponse, 13, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	calculate the checksum */
	unsigned short calculatedChecksum = 0;

	//checksum is bytes 2 through 8 (skip over the 0xAA from the base station and the Start of Packet byte)
	for(int i = 2; i <= 8; i++)
	{
		//add each byte to the calculated checksum
		calculatedChecksum += ucResponse[i];
	}

	if (calculatedChecksum != ((ucResponse[11] << 8) + ucResponse[12]))
	{
		/*	append to the command description	*/
		strFormat.Format("A bad checksum was calculated on the read from node %i\r\nReturned Checksum: %#04x\r\nCalculated Checksum: %#04x", usNode, (ucResponse[11] << 8) + ucResponse[12], calculatedChecksum);
		strCmd_info[iPos] = strFormat;
		return ERROR_BAD_CHECKSUM;
	}

	/*	append to the command description	*/
	if (ucResponse[7] != 0x00 && ucResponse[8] != 0x04 && ucResponse[6] != 0x02) 
	{
		/*	append to the command description	*/
		strFormat.Format("Failed write command with a bad response.");
		strCmd_info[iPos] = strFormat;
		return ERROR_BAD_CHECKSUM;
	}

	/*	append to the command description	*/
	strFormat.Format("Successful write attempt to write %i to EEPROM location %i on node %i", usValue, usLocation, usNode);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_page_download
*	nd_page_download attempts to download a page, usPage, on a node, usNode, over the port
*	opened by hPort.
*	Command Format
*		Length:		5 bytes
*		Byte 1:		0x05
*		Byte 2:		MSB of the node's address
*		Byte 3:		LSB of the node's address
*		Byte 4:		MSB of page to download
*		Byte 5:		LSB of page to download
*	Good Response:
*		Length:		267 bytes
*		Byte 1:		0x05
*		Byte 2-265:	Incoming data
*		Byte 266:	MSB of checksum
*		Byte 267:	LSB of checksum
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to download from
*			usPage is an unsigned short holding the page number to download
*			usResponse is an array of length 132 that will hold all the incoming data
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_page_download(/* in */ HANDLE hPort,
					 /* in */ unsigned short usNode,
					 /* in */ unsigned short usPage,
					 /* out */ unsigned short usResponse[])
{
	unsigned char	ucCmd[5] =	{	0x05,			//	page download command byte
									usNode >> 8,	//	MSB of node address
									usNode & 0xFF,	//	LSB of node address
									usPage >> 8,	//	MSB of page to download
									usPage & 0xFF	//	LSB of page to download
								};
	unsigned char ucResponse[267];		//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	unsigned short usChecksum = 0;		//	calculated checksum for the downloaded page
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 5, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to download page %i on node %i\r\nCommand Byte: %#04x\tMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nMSB Page: %#04x\tLSB Page: %#04x",
		usPage, usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3], ucCmd[4]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, ucResponse, 267, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	calculate the checksum	*/
	for (int i = 1; i < 265; i++)
		usChecksum += ucResponse[i];
	if ((usChecksum & 0xFFFF) != (ucResponse[265] << 8) + ucResponse[266]) {
		/*	append to the command description	*/
		strFormat.Format("A bad checksum was returned for downloading page %i on node %i\r\nReturned Checksum: %#04x\r\nCalculated Checksum: %#04x", 
			usPage, usNode, (ucResponse[265] << 8) + ucResponse[266], usChecksum & 0xFFFF);
		/*	the return will take up multiple lines so put the command information in for every line	*/
		
		int iSize = static_cast<int>(strCmd_info.GetCount());
		for (int i = iSize; i <= iPos; i++) {
			crCmd_color[i] = COLOR_RECIEVE;
			strCmd_info[i] = strFormat;
		}
		return ERROR_BAD_CHECKSUM;
	}

	/*	passed checksum so store the result in the usResponse array	*/
	for (int i = 0; i < 132; i++)
		usResponse[i] = (ucResponse[1 + i * 2] << 8) + ucResponse[2 + i * 2];

	/*	append to the command description	*/
	strFormat.Format("Successful page download attempt for page %i on node %i\r\nChecksum: %#04x", 
		usPage, usNode, (ucResponse[265] << 8) + ucResponse[266]);
	/*	the return will take up multiple lines so put the command information in for every line	*/
	int iSize = static_cast<int>(strCmd_info.GetCount());
	for (int i = iSize; i <= iPos; i++)
		strCmd_info[i] = strFormat;

	return 0;
}

/***********************
*	nd_trigger
*	nd_trigger triggers a node, usNode, with an id value, ucID, over the port
*	opened by hPort
*	Command Format
*		Length:		4 bytes
*		Byte 1:		0x0C
*		Byte 2:		MSB of the node's address
*		Byte 3:		LSB of the node's address
*		Byte 4:		Trigger ID value
*	Good Response:
*		Length:		1 byte
*		Byte 1:		0x0C
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to download from
*			ucID is the ID value for the trigger
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_trigger(/* in */ HANDLE hPort,
				/* in */ unsigned short usNode,
				/* in */ unsigned char ucID)
{
	unsigned char ucCmd[4] =	{	0x0C,			//	trigger id command byte
									usNode >> 8,	//	MSB of the node's address
									usNode & 0xFF,	//	LSB of the node's address
									ucID			//	ID of the node
								};
	unsigned char ucResponse;		//	the value returned from the trigger
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 4, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to start a trigger with ID %i on node %i.\r\nCommand Byte: %#04x\tMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nTrigger ID: %#04x",
		ucID, usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	check for the correct return value	*/
	if (ucResponse != 0x0C) {
		/*	append to the command description	*/
		strFormat.Format("Failed with a bad response to start a trigger with ID %i on node %i\r\nResponse: %#04x", ucID, usNode, ucResponse);
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("Successful attempt to trigger with ID %i on node %i.", ucID, usNode);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_sleep
*	nd_sleep puts a node, usNode, to sleep.
*	NOTE: The wake command is a ping command sent continuously.  This can be accomplished
*			with a for, while, do while, etc. loop and checking the response on each ping
*			attempt.
*	Command Format
*		Length:		3 bytes
*		Byte 1:		0x32
*		Byte 2:		MSB of the node's address
*		Byte 3:		LSB of the node's address
*	Good Response:
*		None
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to sleep
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_sleep(/* in */ HANDLE hPort,
			 /* in */ unsigned short usNode)
{
	unsigned char ucCmd[3] =	{	0x32,			//	sleep command byte
									usNode >> 8,	//	MSB of the node's address
									usNode & 0xFF	//	LSB of the node's address
								};
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 3, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to sleep node %i.\r\nCommand Byte: %#04x\tMSB Node Address: %#04x\tLSB Node Address: %#04x",
		usNode, ucCmd[0], ucCmd[1], ucCmd[2]);
	strCmd_info[iPos] = strFormat;

	/*	there is no response from the serial port	*/

	return 0;
}

/***********************
*	nd_erase
*	nd_erase erases a node, usNode.
*	Command Format
*		Length:		7 bytes
*		Byte 1:		0x32
*		Byte 2:		MSB of the node's address
*		Byte 3:		LSB of the node's address
*		Byte 4:		0x08
*		Byte 5:		0x10
*		Byte 6:		0x0C
*		Byte 7:		0xFF
*	Good Response:
*		Length:		1 byte
*		Byte 1:		0x06
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to erase
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_erase(/* in */ HANDLE hPort,
				/* in */ unsigned short usNode)
{
	unsigned char ucCmd[7] =	{	0x06,			//	erase command byte
									usNode >> 8,	//	MSB of the node's address
									usNode & 0xFF,	//	LSB of the node's address
									0x08,			//	command byte
									0x10,			//	command byte
									0x0C,			//	command byte
									0xFF			//	command byte
								};
	unsigned char ucResponse;		//	the value returned from the trigger
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 7, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to erase node %i.\r\nCommand Byte: %#04x\tMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Bytes: %#04x %#04x %#04x %#04x",
		usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3], ucCmd[4], ucCmd[5], ucCmd[6]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	check for the correct return value	*/
	if (ucResponse != 0x06) {
		/*	append to the command description	*/
		strFormat.Format("Failed attempt to erase node %i with an incorrect response.\r\nResponse: %#04x", usNode, ucResponse);
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("Successful attempt to erase node %i.", usNode);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_stream_init
*	nd_stream_init initiates a stream on a node, usNode, with an 8 bit checksum.
*	Command Format
*		Length:		3
*		Byte 1:		0x38
*		Byte 2:		MSB of Node Address
*		Byte 3:		LSB of Node Address
*	Good Response:
*		Length:		1 byte
*		Byte 1:		0x06
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to initiate a stream on
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_stream_init(/* in */ HANDLE hPort,
					/* in */ unsigned short usNode)
{
	unsigned char ucCmd[3] =	{	0x38,			//	stream command byte
									usNode >> 8,	//	MSB of the node's address
									usNode & 0xFF,	//	LSB of the node's address
								};
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 3, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to stream node %i.\r\nCommand Byte: %#04x\tMSB Node Address: %#04x\tLSB Node Address: %#04x",
		usNode, ucCmd[0], ucCmd[1], ucCmd[2]);
	strCmd_info[iPos] = strFormat;

	/*	align the stream to the first sweep by dumping all incoming garbage and
	*	finding the very first 0xFF	*/
	unsigned char ucGarbage = 0, ucPrev_garbage = 0x01;
	unsigned int uiAA_counter = 0;
	DWORD dwResult_len = 0;
	do
	{
		if ((iResult = port_read(hPort, &ucGarbage, 1, dwBytes_read, iPos)) != 0) {	//	do error correction on this
			return -1;
		};	
		if (ucPrev_garbage != 0xAA)	uiAA_counter = 0;
		else	ucPrev_garbage = ucGarbage;

		if (ucGarbage == 0xAA) {
			if (++uiAA_counter > 4) 		// 5 continuous 0xAA's is considered the end of a stream
				return ERROR_END_OF_STREAM;	// end of stream flags encountered
			
			strFormat.Format("Possible end of stream flag.\r\nByte: %#04x\r\n", ucGarbage);
			strCmd_info[iPos] = strFormat;
		}
		else if (ucGarbage == 0xFF) {	// 0xFF is the flag for the beginning of a sweep
			strFormat.Format("Initial sweep flag byte.\r\nByte: %#04x\r\n", ucGarbage);
			strCmd_info[iPos] = strFormat;
		}
		else {
			strFormat.Format("Garbage byte from stream initialization.\r\nByte: %#04x\r\n", ucGarbage);
			strCmd_info[iPos] = strFormat;
		}
	} while (ucGarbage != 0xFF);
	return 0;
}

/***********************
*	nd_stream_sweep
*	nd_stream_sweep retrieves a sweep from the streaming node via the port opened by
*	the handle, hPort.
*	Command Format
*		Length:		0
*	Good Response:
*		Length:			Num Channels * 2 + 1 byte checksum + 1 byte next sweep flag
*		Byte 1:			MSB of 1st channel's data
*		Byte 2:			LSB of 1st channel's data
*		Byte 3:			MSB of 2nd channel's data
*		...
*		Byte N - 2:		Checksum of bytes 1 through N - 3
*		Byte N - 1:		Next sweep flag
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			pusResponse is a legitimate pointer of length equal to the number of active
*				channels on the streaming node signified by ucNum_channels which holds
*				the sweep returned by the serial port
*			ucNum_channels is an unsigned character that holds the number of active channels
*				on the streaming node.  This is also equal to the length of pusResponse.
*	POST:	if successful, 0 is returned and pusResponse holds the next sweep in the stream
*			otherwise, some error code
***********************/
int nd_stream_sweep(/* in */ HANDLE hPort,
					/* in/out */ unsigned short *pusResponse,
					/* in */ unsigned char ucNum_channels)
{
	dynamic_buffer<unsigned char> ucResponse(ucNum_channels * 2 + 2);	//	2 bytes for each channel, 1 for the checksum, 1 for next 0xFF flag
	DWORD dwBytes_read = 0;			//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat,				//	formatted string for output to command information window
			strFormat_temp;			//	temporary string for assistance with formatting output text
	/*	attempt to read 1 sweep	*/
	if ((iResult = port_read(hPort, ucResponse, ucNum_channels * 2 + 2, dwBytes_read, iPos)) != 0) {
		/*	check for the end of stream responses	*/
		bool isEnd = true;
		for (DWORD i = 0; i < dwBytes_read; i++)
			if (ucResponse[i] != 0xAA)
				isEnd = false;
		if (isEnd) {
			strFormat = "End of stream flags.";
			strCmd_info[iPos] = strFormat;
			return ERROR_END_OF_STREAM;
		}
		else {
			strFormat = "Unknown packet format.";
			strCmd_info[iPos] = strFormat;
		}
		return ERROR_MISALIGNED_SWEEP_PACKET;	//	add errors and check for 0xAA's
	}

	/*	This long block of code is for adding command information to the console and
	*	has nothing to do with the 2400 Mhz protocol.	*/
	strFormat.Format("Successful sweep recieved.\r\n");
	for (int i = 0; i < ucNum_channels; i++) {
		strFormat_temp.Format("MSB Channel: %#04x\tLSB Channel: %#04x\tCalculated Value: %i\r\n",
			ucResponse[i * 2], ucResponse[i * 2 + 1], ((ucResponse[i * 2] << 8) + ucResponse[i * 2 + 1]) >> 1);
		strFormat += strFormat_temp;
	}
	strFormat_temp.Format("Checksum: %#04x\t\t", ucResponse[ucNum_channels * 2]);
	strFormat += strFormat_temp;
	if (ucResponse[ucNum_channels * 2 + 1] == 0xFF)
		strFormat_temp.Format("Sweep Flag: %#04x\r\n", ucResponse[ucNum_channels * 2 + 1]);
	else if (ucResponse[ucNum_channels * 2 + 1] == 0xAA)
		strFormat_temp.Format("End of Stream Flag: %#04x\r\n", ucResponse[ucNum_channels * 2 + 1]);
	else
		strFormat_temp.Format("Unknown Trailing Byte: %#04x\r\n", ucResponse[ucNum_channels * 2 + 1]);
	strFormat += strFormat_temp;
	strCmd_info[iPos] = strFormat;

	/*	calculate the checksum to make sure it's right	*/
	unsigned char ucChecksum = 0;
	for (int i = 0; i < ucNum_channels * 2; i++)
		ucChecksum += ucResponse[i];

	/*	check to make sure calculated and returned checksums match	*/
	if (ucChecksum != ucResponse[ucNum_channels * 2])
		return ERROR_BAD_CHECKSUM;	//	bad checksum

	/*	put in the legitimate data	*/
	for (int i = 0; i < ucNum_channels; i++)
		pusResponse[i] = (ucResponse[i * 2] << 8) + ucResponse[i * 2 + 1];
	return 0;
}

/***********************
*	nd_low_duty_cycle_init
*	nd_low_duty_cycle_init attempts to initiate a low duty cycle on a node, usNode, over the port, hPort.
*	Command Format
*		Length:		10 bytes
*		Byte 1:		0xAA		SOP
*		Byte 2:		0x05		Message Type
*		Byte 3:		0x00		Address Mode
*		Byte 4:		MSB of Node Address
*		Byte 5:		LSB of Node Address
*		Byte 6:		0x02
*		Byte 7:		0x00		Command Byte
*		Byte 8:		0x38		Command Byte
*		Byte 9:		MSB Checksum
*		Byte 10:	LSB Checksum
*	Good Response:
*		Length:		1 Byte
*		Byte 1:		0xAA		Valid packet response from base station
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to ping
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_low_duty_cycle_init(/* in */ HANDLE hPort,
						   /* in */ unsigned short usNode)
{

	unsigned char ucCmd[10] = {	0xAA,			//	SOP 
								0x05,			//	Message Type
								0x00,			//	Address Mode
								usNode >> 8,	//	MSB of Node Address
								usNode & 0xFF,	//	LSB of Node Address
								0x02,			//	Command Length
								0x00,			//	MSB of Command Byte
								0x38,			//	LSB of Command Byte
								(0x3F + (usNode >> 8) + (usNode & 0xFF)) >> 8,		//	MSB of the checksum
								(0x3F + (usNode >> 8) + (usNode & 0xFF)) & 0xFF		//	LSB of the checksum
	};

	unsigned char ucResponse;		//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 10, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to start low duty cycle on node %i\r\nSOP: %#04x\t\t\tMessage Type: %#04x\t\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tMSB of Command Byte: %#04x\tLSB of Command Byte: %#04x\r\nMSB of Checksum: %#04x\t\tLSB of Checksum: %#04x",
			usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3], ucCmd[4], ucCmd[5], ucCmd[6], ucCmd[7],
			ucCmd[8], ucCmd[9]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	the only legitimate response is a 0xAA	*/
	if (ucResponse != 0xAA) {
		/*	append to the command description	*/
		strFormat.Format("Failed to start low duty cycle on node %i\r\nResponse: %#04x", usNode, ucResponse);
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("Low duty cycle attempt successful for node %i\r\nResponse: %#04x", usNode, ucResponse);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_low_duty_cycle_packet
*	nd_low_duty_cycle_packet attempts to receive a low duty cycle from a node, usNode, over the port, hPort.
*	Good Response:
*		Length:			XX Bytes
*		Byte 1:			0xAA		SOP
*		Byte 2:			0x07		Message Type
*		Byte 3:			0x04		Address Mode
*		Byte 4:			0xXX		MSB Received From Address
*		Byte 5:			0xXX		LSB	Received From Address
*		Byte 6:			0xXX		Packet Length (does not include LQI or RSSI)
*		Byte 7:			0x02		Application identifier
*		Byte 8:			0xXX		Channel Mask
*		Byte 9:			0xXX		Sampling rate of the packet
*		Byte 10:		0x01		Data Type - Always 2 byte integer
*		Byte 11:		0xXX		MSB Timer Tick
*		Byte 12:		0xXX		LSB Timer Tick
*		Byte 13:		0xXX		MSB of first channel of data
*		Byte 14:		0xXX		LSB of first channel of data
*		Byte 15:		to #						MSB/LSB of the rest of the channels of data if there are any.  Thus, the total
*						channels *					number of bytes to read of channel data is the number of channels active
*						2 minutes 3	0xXX			in the channel mask * 2.
*		
*		Byte N - 3:		0xXX		LQI
*		Byte N - 2:		0xXX		RSSI
*		Byte N - 1:		0xXX		Packet checksum MSB
*		Byte N:			0xXX		Packet checksum LSB
*
*	0xXX - Variable value
*	Checksum = Sum of bytes 2 through N - 4 minus the LQI and RSSI = ((Byte N) * 256 + Byte N - 1) & 0xFFFF
*	The LQI and RSSI are not part of the checksum
***********************/
int nd_low_duty_cycle_packet(/* in */ HANDLE hPort,
							 /*	in	*/ unsigned char ucNumChannels)
{
	DWORD dwPacketLen = ucNumChannels * 2 + 16;	            //	total length of the packet
	dynamic_buffer<unsigned char> ucResponse(dwPacketLen);	//	holds the low duty cycle packet
	DWORD dwBytes_read = 0;			//	number of bytes read from the port
	int iResult,						//	return value of port read/write functions
		iPos;							//	position in the console that the command was written to
	CString strFormat;					//	formatted string for output to command information window

	if ((iResult = port_read(hPort, ucResponse, dwPacketLen, dwBytes_read, iPos)) != 0) {
		return iResult;
	}

	strFormat.Format("Low duty cycle packet received.\r\nSOP: %#04x\t\t\tMessage Type: %#04x\t\t\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tApplication Identifier: %#04x\t\tChannel Mask: %#04x\r\nSample Rate: %#04x\t\tData Type: %#04x\r\nMSB Timer Tick: %#04x\t\tLSB Timer Tick: %#04x\r\n", 
		ucResponse[0], ucResponse[1], ucResponse[2], ucResponse[3], ucResponse[4], ucResponse[5], ucResponse[6], ucResponse[7],
		ucResponse[8], ucResponse[9], ucResponse[10], ucResponse[11]);
	CString strTemp;
	/*	Even though we already know the number of channels, use the channel mask to decide which data goes to
	*	which channel.	*/
	int iResponseLoc = 12;	//	position in the ucResponse array for channel data
	for (int iCurChan = 0; iCurChan < 8; iCurChan++)
		if ((ucResponse[7] >> iCurChan) & 1) {
			strTemp.Format("MSB Channel %i Data: %#04x\tLSB Channel %i Data: %#04x\t\tCalculated Value: %i\r\n", iCurChan + 1, ucResponse[iResponseLoc], iCurChan + 1, ucResponse[iResponseLoc + 1], ((ucResponse[iResponseLoc] << 8) + ucResponse[iResponseLoc + 1]) >> 1);
			iResponseLoc += 2;
			strFormat += strTemp;
		}
	
	strTemp.Format("LQI: %#04x\t\t\tRSSI: %#04x\t\tMSB Checksum: %#04x\tLSB Checksum: %#04x", ucResponse[iResponseLoc], ucResponse[iResponseLoc + 1], ucResponse[iResponseLoc + 2], ucResponse[iResponseLoc + 3]);
	strFormat += strTemp;
	
	int iSize = static_cast<int>(strCmd_info.GetSize());
	do {
		strCmd_info[iSize] = strFormat;
	} while (iSize++ < iPos);

	return 0;
}

/***********************
*	nd_sync_sampling_packet
*	nd_sync_sampling_packet attempts to receive a synchronized sampling packet from a node, usNode, over the port, hPort.
*	Good Response:
*		Length:			XX Bytes
*		Byte 1:			0xAA		SOP
*		Byte 2:			0x07		Message Type
*		Byte 3:			0x09		Address Mode (09/0A stands for synchronized buffered packet)
*		Byte 4:			0xXX		Node Address MSB
*		Byte 5:			0xXX		Node Address LSB
*		Byte 6:			0xXX		Packet Length (does not include LQI or RSSI)
*		Byte 7:			0x02		Application identifier
*		Byte 8:			0xXX		Channel Mask
*		Byte 9:			0xXX		Sampling rate of the packet
*		Byte 10:		0xXX		Data Type - Either 0x01 for 2 byte integer or 0x02 for 4-byte float
*		Byte 11:		0xXX		MSB Timer Tick
*		Byte 12:		0xXX		LSB Timer Tick
*		Byte 13:		0xXX		Timestamp Seconds byte1 (MSB)
*		Byte 14:		0xXX		Timestamp Seconds byte2
*		Byte 15:		0xXX		Timestamp Seconds byte3
*		Byte 16:		0xXX		Timestamp Seconds byte4	(LSB)
*		Byte 17:		0xXX		Timestamp Nanoseconds byte1 (MSB)
*		Byte 18:		0xXX		Timestamp Nanoseconds byte2 
*		Byte 19:		0xXX		Timestamp Nanoseconds byte3 
*		Byte 20:		0xXX		Timestamp Nanoseconds byte4 (LSB)
*		Byte 21:		0xXX		MSB of first channel of data
*		Byte 22:		0xXX		LSB of first channel of data
*		Byte 23to(N-4):	0xXX		MSB/LSB of the rest of the channels of data if there are any.  Thus, the total
*									number of bytes to read of channel data is the number of channels active
*									in the channel mask * 2.
*		
*		Byte N - 3:		0xXX		LQI
*		Byte N - 2:		0xXX		RSSI
*		Byte N - 1:		0xXX		Packet checksum MSB
*		Byte N:			0xXX		Packet checksum LSB
*
*	0xXX - Variable value
*	Checksum = Sum of bytes 2 through N - 4 minus the LQI and RSSI = ((Byte N) * 256 + Byte N - 1) & 0xFFFF
*	The LQI and RSSI are not part of the checksum
***********************/
int nd_sync_sampling_packet(HANDLE hPort)
{
	const int BYTES_BEFORE_CHNL_DATA = 14;	//bytes between payload and channel data

	unsigned char ucSOPResponse = 0;								//	SOP response byte
	dynamic_buffer<unsigned char> ucHeaderResponse(5);				//	holds the sync sampling header packet
	DWORD dwBytes_read = 0;											//	number of bytes read from the port
	int iResult;													//	return value of port read/write functions
	int iPos;														//	position in the console that the command was written to
	CString strFormat;												//	formatted string for output to command information window

	//look for the start of packet indicator
	while(ucSOPResponse != 0xAA)
	{
		//read 1 byte at a time looking for the SOP
		if ((iResult = port_read(hPort, &ucSOPResponse, 1, dwBytes_read, iPos)) != 0) 
		{
			return iResult;
		}
	}

	//we found an 0xAA at this point, so read in the header data
	if ((iResult = port_read(hPort, ucHeaderResponse, 5, dwBytes_read, iPos)) != 0) 
	{
		return iResult;
	}

	strFormat.Format("Synchronized Sampling packet received.\r\nSOP: %#04x\t\t\tMessage Type: %#04x\t\t\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\r\n", 
		ucSOPResponse, ucHeaderResponse[0], ucHeaderResponse[1], ucHeaderResponse[2], ucHeaderResponse[3], ucHeaderResponse[4]);

	//get the length of the data
	unsigned char usPayloadLength = ucHeaderResponse[4];

	dynamic_buffer<unsigned char> ucDataResponse(usPayloadLength + 4);//	holds the sync sampling data packet

	if ((iResult = port_read(hPort, ucDataResponse, usPayloadLength + 4, dwBytes_read, iPos)) != 0)
	{
		return iResult;
	}

	unsigned char ucChannelMask = ucDataResponse[1];
	unsigned char ucNumChannels = 0;

	unsigned char ActiveChannels[8];
	for (int iCurChan = 0; iCurChan < 8; iCurChan++)
	{
		//loop through the bits to see which channels are enabled
		if ((ucChannelMask >> iCurChan) & 1)
		{
			ActiveChannels[ucNumChannels] = iCurChan + 1;
			ucNumChannels++;
		}
	}

	unsigned char ucDataType = ucDataResponse[3];
	unsigned char ucLengthMultiplier = 2;

	if(ucDataType == 1)
		ucLengthMultiplier = 2;//2 byte integer format
	else if(ucDataType == 2)
		ucLengthMultiplier = 4;//4 byte float format
	
	//get the timestamp values (seconds and nanoseconds)
	WORD lowSec		= MAKEWORD(ucDataResponse[9], ucDataResponse[8]);
	WORD highSec	= MAKEWORD(ucDataResponse[7], ucDataResponse[6]);
	INT64 seconds	= MAKELONG(lowSec, highSec);
	
	WORD lowNano	= MAKEWORD(ucDataResponse[13], ucDataResponse[12]);
	WORD highNano	= MAKEWORD(ucDataResponse[11], ucDataResponse[10]);
	INT64 nanoSec	= MAKELONG(lowNano, highNano);

	//timestamp in nanoseconds (convert seconds to nanoseconds and add the nanoseconds in)
	INT64 realTimeStamp = (seconds * 1000000000L) + nanoSec;

	strFormat.AppendFormat("Timestamp: %lli nanoseconds\r\n", realTimeStamp);

	unsigned char ucSampleRateValue = ucDataResponse[2];
	double dSampleRateInc;

	for(int i = 0; i < N24_NUM_SYNC_SAMPLING_RATES; i++)
	{
		if(ucSampleRateValue == ldrSyncSamplingRates[i].m_ucValue)
		{
			dSampleRateInc = ldrSyncSamplingRates[i].m_dTime_inc;
			break;
		}
	}

	int iOffset		= ((usPayloadLength - BYTES_BEFORE_CHNL_DATA) / ucNumChannels);
	int iSweepCount = iOffset / ucLengthMultiplier;

	if(ucLengthMultiplier == 2)//2 byte int
	{
		INT64 increment = (dSampleRateInc * 1000000000);

		for(int iSweep = 0; iSweep < iSweepCount; iSweep++)
		{
			int iSweepOffset = iSweep * ucLengthMultiplier * ucNumChannels;

			for(int iChannelIndex = 0; iChannelIndex < ucNumChannels; iChannelIndex++)
			{
				int iChannelOffset = iChannelIndex * ucLengthMultiplier;

				//here we have the actual channel data
				unsigned char ucCurrentChannel = ActiveChannels[iChannelIndex];
				strFormat.AppendFormat("MSB Channel %i Data: %#04x\tLSB Channel %i Data: %#04x\t\tCalculated Value: %i\r\n",
					ucCurrentChannel, ucDataResponse[6 + 8 + iChannelOffset + iSweepOffset], ucCurrentChannel, ucDataResponse[6 + 8 + 1 + iChannelOffset + iSweepOffset], ((ucDataResponse[6 + 8 + iChannelOffset + iSweepOffset] << 8) + (ucDataResponse[6 + 8 + 1 + iChannelOffset + iSweepOffset])) >> 1);
			}

			//increment the timestamp ourselves for each sweep
			realTimeStamp += increment;
			
			strFormat.AppendFormat("Timestamp: %lli nanoseconds\r\n", realTimeStamp);
		}
	}
	else if(ucLengthMultiplier == 4)//4 byte float
	{
		unsigned char *ucBuffer;

		INT64 increment = (dSampleRateInc * 1000000000);

		for(int iSweep = 0; iSweep < iSweepCount; iSweep++)
		{
			int iSweepOffset = iSweep * ucLengthMultiplier * ucNumChannels;

			for(int iChannelIndex = 0; iChannelIndex < ucNumChannels; iChannelIndex++)
			{
				int iChannelOffset = iChannelIndex * ucLengthMultiplier;

				//here we have the actual channel data
				float *fData;
				
				ucBuffer	= (unsigned char*)&fData[0];
				ucBuffer[0] = ucDataResponse[9 + 8 + iChannelOffset + iSweepOffset];
				ucBuffer[1] = ucDataResponse[8 + 8 + iChannelOffset + iSweepOffset];
				ucBuffer[2] = ucDataResponse[7 + 8 + iChannelOffset + iSweepOffset];
				ucBuffer[3] = ucDataResponse[6 + 8 + iChannelOffset + iSweepOffset];

				unsigned char ucCurrentChannel = ActiveChannels[iChannelIndex];
				strFormat.AppendFormat("Channel %i Float Value: %f\r\n", ucCurrentChannel, fData);
			}

			//increment the timestamp ourselves for each sweep
			realTimeStamp += increment;
			strFormat.AppendFormat("Timestamp: %li nanoseconds\r\n", realTimeStamp);
		}
	}

	
	int iSize = static_cast<int>(strCmd_info.GetSize());
	do 
	{
		strCmd_info[iSize] = strFormat;
	} while (iSize++ < iPos);

	return 0;
}

/***********************
*	nd_high_speed_stream_init
*	nd_high_speed_stream_init attempts to initiate a high speed stream on a node, usNode, over the port, hPort.
*	Command Format
*		Length:		10 bytes
*		Byte 1:		0xAA		SOP
*		Byte 2:		0x05		Message Type
*		Byte 3:		0x00		Address Mode
*		Byte 4:		MSB of Node Address
*		Byte 5:		LSB of Node Address
*		Byte 6:		0x02
*		Byte 7:		0x00		Command Byte
*		Byte 8:		0x39		Command Byte
*		Byte 9:		MSB Checksum
*		Byte 10:	LSB Checksum
*	Good Response:
*		Length:		1 Byte
*		Byte 1:		0xAA		Valid packet response from base station
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to ping
*	POST:	if successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_high_speed_stream_init(/* in */ HANDLE hPort,
						   /* in */ unsigned short usNode)
{
	unsigned char ucCmd[10] = {	0xAA,			//	SOP 
								0x05,			//	Message Type
								0x00,			//	Address Mode
								usNode >> 8,	//	MSB of Node Address
								usNode & 0xFF,	//	LSB of Node Address
								0x02,			//	Command Length
								0x00,			//	MSB of Command Byte
								0x39,			//	LSB of Command Byte
								(0x40 + (usNode >> 8) + (usNode & 0xFF)) >> 8,		//	MSB of the checksum
								(0x40 + (usNode >> 8) + (usNode & 0xFF)) & 0xFF		//	LSB of the checksum
	};

	unsigned char ucResponse;		//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 10, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to start high speed streaming on node %i\r\nSOP: %#04x\t\t\tMessage Type: %#04x\t\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tMSB of Command Byte: %#04x\tLSB of Command Byte: %#04x\r\nMSB of Checksum: %#04x\t\tLSB of Checksum: %#04x",
			usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3], ucCmd[4], ucCmd[5], ucCmd[6], ucCmd[7],
			ucCmd[8], ucCmd[9]);
	strCmd_info[iPos] = strFormat;

	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	the only legitimate response is an 0xAA	*/
	if (ucResponse != 0xAA) {
		/*	append to the command description	*/
		strFormat.Format("Failed to start high speed stream on node %i\r\nResponse: %#04x", usNode, ucResponse);
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("High speed stream attempt successful for node %i\r\nResponse: %#04x", usNode, ucResponse);
	strCmd_info[iPos] = strFormat;

	return 0;
}


/***********************
*	nd_high_speed_stream_packet
*	nd_high_speed_stream_packet attempts to receive a high speed packet from a node, usNode, over the port, hPort.
*	Good Response:
*		Length:			XX Bytes
*		Byte 1:			0xAA		SOP
*		Byte 2:			0x01		Message Type
*		Byte 3:			0x05		Address Mode
*
*		Byte 4:			0xXX		MSB Received From Address
*		Byte 5:			0xXX		LSB	Received From Address
*
*		Byte 6:			0xXX		Packet Length (does not include LQI or RSSI)
*		Byte 7:			0x02		Application identifier
*		Byte 8:			0xXX		Channel Mask   
*		Byte 9:			0xFF		Stream rate of the packet, fixed at 0xFF, the actual rate is dermined by the number of channels
*		Byte 10: 		0xXX		Number of samples
*
*		Byte 11:		0xXX		Timer Value (4 bytes)
*		Byte 12:		0xXX		
*		Byte 13:		0xXX		
*		Byte 14:		0xXX		
*
*       Byte 15 to N-4: 0xXX       Sample Data 
*		for each sample
*		   for each channel
*		      Byte 15 + (S*Channels + C)*2: MSB of Sample(S) channel(C)
*			  Byte 16 + (S*Channels + C)*2:	LSB of Sample(S) channel(C)
*		
*		Byte N - 3:		0xXX		LQI
*		Byte N - 2:		0xXX		RSSI
*
*		Byte N - 1:		0xXX		Packet checksum MSB
*		Byte N:			0xXX		Packet checksum LSB
*
*	0xXX - Variable value
*	Checksum = Sum of bytes 2 through N - 4(exclude LQI, RSSI, and cheksum) = ((Byte N) * 256 + Byte N - 1) & 0xFFFF
*	The LQI and RSSI are not part of the checksum
***********************/
int nd_high_speed_stream_packet(/* in */ HANDLE hPort)
{
	DWORD dwBytes_read = 0;			//	number of bytes read from the port
	int iResult,						//	return value of port read/write functions
		iPos;							//	position in the console that the command was written to
	CString strFormat;					//	formatted string for output to command information window

	unsigned char ucPacketHeader[14] = {0};

	/* Read the header of the packet to detrmine payload size */
	if ((iResult = port_read(hPort, ucPacketHeader, 14, dwBytes_read, iPos)) != 0) {
		return iResult;
	}

	strFormat.Format("High speed stream packet received.\r\n"
	                 "SOP: %#04x\t\tMessage Type: %#04x\tAddress Mode: %#04x\r\n"
	                 "MSB Node Address: %#04x\tLSB Node Address: %#04x\tCommand Length: %#04x\r\n"
					 "Application Identifier: %#04x\tChannel Mask: %#04x\tStream Rate: %#04x\r\n"
					 "Sample Count: %#04x\tTimer Value: %#04x %#04x %#04x %#04x\r\n",
		ucPacketHeader[0], ucPacketHeader[1],  ucPacketHeader[2],
		ucPacketHeader[3], ucPacketHeader[4],  ucPacketHeader[5],
		ucPacketHeader[6], ucPacketHeader[7],  ucPacketHeader[8],
		ucPacketHeader[9], ucPacketHeader[10], ucPacketHeader[11], ucPacketHeader[12],  ucPacketHeader[13]);
	CString strTemp;

	unsigned char ucDataLength  = ucPacketHeader[5];
	unsigned char ucSampleCount = ucPacketHeader[9];
	unsigned char ucChannelMask = ucPacketHeader[7];

	/* Part of the packets data was read as part of the header */
	ucDataLength -= 8; // 8=>Application Identifier(1) + Channel Mask(1) + Stream Rate(1) + Sample Count(1) + Timer Value(4)

	/* also read RSSI, LQI and the checksum wich are not included in the packet data length */
	ucDataLength += 4; // 4 => RSSI(1 byte) + LQI(1 byte) + Checksum(2 bytes)
	
	/* read the rest of the Packet */
	dynamic_buffer<BYTE> responseData(ucDataLength);
	if ((iResult = port_read(hPort, responseData, ucDataLength, dwBytes_read, iPos)) != 0) {
		return iResult;
	}

	/* create a header for the detailed display of the data */
	CString strHeaderLine1 = "Sample \t";
	CString strHeaderLine2 = "       \t";
	for (int iCurChan = 0; iCurChan < 8; iCurChan++)
	{
		if ((ucChannelMask >> iCurChan) & 1)
		{
			strHeaderLine1.AppendFormat("     Channel %d\t\t", iCurChan+1);
			strHeaderLine2.Append("MSB   LSB   (Value)\t\t");
		}
	}

	/* add the header to the description */
	strFormat += strHeaderLine1;
	strFormat += "\r\n";
	strFormat += strHeaderLine2;
	strFormat += "\r\n";

	/* count the channels */
	int iChannelCount = 0;
	for (int iCurChan = 0; iCurChan < 8; iCurChan++)
	{
		if ((ucChannelMask >> iCurChan) & 1)
		{
			iChannelCount++;
		}
	}

	/* proccess each sample, placing the data in the description text */
	int iResponseLoc = 0;
	for(int iSample = 0; iSample < ucSampleCount; iSample++)
	{
		strTemp.Format("%d\t", iSample+1);

		//	Use the channel mask to decide which data goes to which channel.
		int channelOffset = 0;
		for (int iCurChan = 0; iCurChan < 8; iCurChan++)
		{
			if ((ucChannelMask >> iCurChan) & 1)
			{
				iResponseLoc = (iSample*iChannelCount + channelOffset)*2;
				int iValue = ((responseData[iResponseLoc] << 8) + responseData[iResponseLoc+1]);
				strTemp.AppendFormat("%#04x  %#04x  (%#05d)     \t", responseData[iResponseLoc], responseData[iResponseLoc+1], (responseData[iResponseLoc] << 8) + responseData[iResponseLoc+1]);
				channelOffset++;
			}
		}

		strFormat += strTemp;
		strFormat += "\r\n";
	}

	//the last 4 bytes are the RSSI, LQI, and checksum
	strTemp.Format("LQI: %#04x\tRSSI: %#04x\r\n", responseData[ucDataLength - 4], responseData[ucDataLength - 3]);
	strFormat += strTemp;
	
	WORD wReturedChecksum = (responseData[ucDataLength - 2] << 8 ) + responseData[ucDataLength -1];

	//calculate the checksum
	WORD wCalculatedChecksum = 0;
	for(int i = 1; i < 14; i++)
	{
		wCalculatedChecksum += ucPacketHeader[i];
	}

	for(int i = 0; i < ucDataLength-4; i++)
	{
		wCalculatedChecksum += responseData[i];
	}

	//out put the returned and calculated checksums, they should always be identical
	ASSERT(wReturedChecksum == wCalculatedChecksum);
	strFormat.AppendFormat("Returned Checksum:\t %#04x\r\n", wReturedChecksum);
	strFormat.AppendFormat("Calculated Checksum:\t %#04x\r\n", wCalculatedChecksum);

	//add the detailed data to each line of the raw output
	int iSize = static_cast<int>(strCmd_info.GetSize());
	do {
		strCmd_info[iSize] = strFormat;
	} while (iSize++ < iPos);

	return 0;
}


/***********************
*	nd_discovery_packet
*	nd_discovery_packet attempts to receive a node discovery packet from any node that sends one over the port, hPort.
*	Good Response:
*		Length:		1 Byte
*		Byte 1:		0xAA		SOP
*		Byte 2:		0x00		Message Type				OR		(Byte 2 & 0x08) == 0x00
*		Byte 3:		0x00		Address Mode				OR		Byte 3 == 0x07
*		Byte 4:		0xXX		MSB Received From Address
*		Byte 5:		0xXX		LSB	Received From Address
*		Byte 6:		0x03		Packet Length
*		Byte 7:		0xXX		Radio Channel
*		Byte 8:		0xXX		MSB of Model Number (see node EEPROM map for valid model numbers in EEPROM 112)
*		Byte 9:		0xXX		LSB of Model Number (see node EEPROM map for valid model numbers in EEPROM 112)
*		Byte 10:	0xXX		LQI
*		Byte 11:	0xXX		RSSI
*		Byte 12:	0xXX		Packet checksum MSB
*		Byte 13:	0xXX		Packet checksum LSB
*
*	0xXX - Variable value
*	Checksum = Sum of bytes 2 through 9 = ((Byte N) * 256 + Byte N - 1) & 0xFFFF
*	The LQI and RSSI are not part of the checksum
***********************/
int nd_discovery_packet(/* in */ HANDLE hPort)
{
	unsigned char	ucResponse[13],		//	buffer to store the discovery packet in
					ucLength = 13;		//	the length of the discovery packet
	DWORD dwBytes_read;
	int iResult,						//	return value of port read/write functions
		iPos;							//	position in the console that the command was written to
	CString strFormat;					//	formatted string for output to command information window

	if ((iResult = port_read(hPort, ucResponse, ucLength, dwBytes_read, iPos)) != 0) {
		return iResult;
	}

	/*	Perform a quick verification of the packet.  This verification only verifies
		the SOP, Message type, Address mode, and packet length.  For a more thorough verification,
		also verify that the radio channels and model numbers are valid.	*/
	if (!(dwBytes_read == 13 && ucResponse[0] == 0xAA && 
			((ucResponse[1] == 0x00 && ucResponse[2] == 0x00) || ((ucResponse[1] & 0x08) == 0x00 && ucResponse[2] == 0x07)) &&
			ucResponse[5] == 0x03)) {
		strFormat.Format("Received data was invalid.\r\n");
		CString strTemp;
		for (DWORD i = 0; i < dwBytes_read; i++) {
			strTemp.Format("Byte %i: %#04x\t\t", i + 1, ucResponse[i]);
			if (i % 3 == 1 && i != 0)
				strTemp += "\r\n";
			strFormat += strTemp;
		}
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	Translate the packet and output the description to the list of packet descriptions.	*/
	strFormat.Format("Node Discovery Packet received.\r\nSOP: %#04x\t\t\tMessage Type: %#04x\t\t\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tRadio Channel: %#04x\r\nMSB Model Number: %#04x\tLSB Model Number: %#04x\r\nLQI: %#04x\t\t\tRSSI: %#04x\r\nMSB Checksum: %#04x\t\tLSB Checksum: %#04x", 
		ucResponse[0], ucResponse[1], ucResponse[2], ucResponse[3], ucResponse[4], ucResponse[5], ucResponse[6], ucResponse[7],
		ucResponse[8], ucResponse[9], ucResponse[10], ucResponse[11], ucResponse[12]);
	strCmd_info[iPos] = strFormat;

	return 0;
}




/***********************
*	nd_kill_node
*	nd_kill_node attempts to stop a node that is in a state that doesn't accept commands. ie streaming, LDC etc
*	Command Format
*		Length:		10 bytes
*		Byte 1:		0xAA		SOP
*		Byte 2:		0xFE		Message Type
*		Byte 3:		0x00		Address Mode
*		Byte 4:		MSB of Node Address
*		Byte 5:		LSB of Node Address
*		Byte 6:		0x02
*		Byte 7:		0x00		Command Byte
*		Byte 8:		0x90		Command Byte
*		Byte 9:		MSB Checksum
*		Byte 10:	LSB Checksum
*	Good Response:
*		Length:		1 Byte
*		Byte 1:		0xAA		Valid packet response from base station
*
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			usNode is an unsigned short holding the node's address to stop
*	POST:	if command is sent successful, 0 is returned
*			otherwise, some error code
*
* Note:Must call nd_kill_node_status to poll the status of the kill node command
* you can also call abort the kill node command by sending any byte to the
* basestation. Pinging the basestation will suffice.
***********************/
int nd_kill_node_init(/* in */ HANDLE hPort,
					  /* in */ unsigned short usNode)
{

	
	

	unsigned char ucCmd[10] = {	0xAA,			//	SOP 
								0xFE,			//	Message Type
								0x00,			//	Address Mode
								usNode >> 8,	//	MSB of Node Address
								usNode & 0xFF,	//	LSB of Node Address
								0x02,			//	Command Length
								0x00,			//	MSB of Command Byte
								0x90,			//	LSB of Command Byte
								(0x190 + (usNode >> 8) + (usNode & 0xFF)) >> 8,		//	MSB of the checksum
								(0x190 + (usNode >> 8) + (usNode & 0xFF)) & 0xFF	//	LSB of the checksum
	};

	unsigned char ucResponse;		//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	// before we send the kill node command send a byte to interupt the base station
	// this will stop it from listening for streaming, HSS or LDC data
	unsigned char ucByte = 0x01;
	if ((iResult = port_write(hPort, &ucByte, 1, dwBytes_written, iPos)) != 0)
		return iResult;
	strCmd_info[iPos] = "Send byte to stop the base station from streaming.";

	//now purge the buffer
	port_purge(hPort); 

	/*	send the command to the serial port	*/
	if ((iResult = port_write(hPort, ucCmd, 10, dwBytes_written, iPos)) != 0)
		return iResult;

	/*	append to the command description	*/
	strFormat.Format("Attempt to kill node on node %i\r\nSOP: %#04x\t\t\tMessage Type: %#04x\t\tAddress Mode: %#04x\r\nMSB Node Address: %#04x\tLSB Node Address: %#04x\r\nCommand Length: %#04x\t\tMSB of Command Byte: %#04x\tLSB of Command Byte: %#04x\r\nMSB of Checksum: %#04x\t\tLSB of Checksum: %#04x",
			usNode, ucCmd[0], ucCmd[1], ucCmd[2], ucCmd[3], ucCmd[4], ucCmd[5], ucCmd[6], ucCmd[7],
			ucCmd[8], ucCmd[9]);
	strCmd_info[iPos] = strFormat;


	/*	read the response from the serial port	*/
	if ((iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos)) != 0)
		return iResult;

	/*	the only legitimate response is a 0xAA	*/
	if (ucResponse != 0xAA) {
		/*	append to the command description	*/
		strFormat.Format("Failed to start kill node on node %i\r\nResponse: %#04x", usNode, ucResponse);
		strCmd_info[iPos] = strFormat;
		return ERROR_INVALID_RESPONSE;
	}

	/*	append to the command description	*/
	strFormat.Format("Kill node command started successful for node %i\r\nResponse: %#04x", usNode, ucResponse);
	strCmd_info[iPos] = strFormat;

	return 0;
}

/***********************
*	nd_kill_node_status
*	nd_kill_node_status waits 100ms for a response from the basestation. 
*	Response:
*		Length:		1 Byte
*		Byte 1:		0x90		The node was successfully stop and is now communicating
*       		    0x21		The kill command was aborted
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			nd_kill_node_init was previously called
*	POST:	if command is sent successful, 0 is returned
*			otherwise, some error code
***********************/
int nd_kill_node_status(/* in */ HANDLE hPort,
					    /* out */ bool& bNodeKilled)
{
	bNodeKilled = false;

	unsigned char ucResponse;		//	holds the response
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	/* set the read timeout to 100 ms */
	port_update_read_timeout(hPort, 100);

	/*	read the response from the serial port	*/
	iResult = port_read(hPort, &ucResponse, 1, dwBytes_read, iPos);
	if(iResult== ERROR_MISMATCHING_BYTE_LENGTH){iResult = 0;}//no response is a valid state since we are polling, waiting for the return
	if(iResult!= 0)
		return iResult;

	/*	did we get any data	*/
	if(dwBytes_read > 0)
	{
        /*	node successfully stopped */
		if (ucResponse == 0x90)
		{
			/*	append to the command description	*/
			strFormat.Format("Stop node successful\r\nResponse: %#04x", ucResponse);
			strCmd_info[iPos] = strFormat;	

			bNodeKilled = true;
		}

		/*	kill node canceled by the user */
		else if(ucResponse == 0x21)
		{
			/*	append to the command description	*/
			strFormat.Format("Stop node aborted\r\nResponse: %#04x", ucResponse);
			strCmd_info[iPos] = strFormat;
			return ERROR_KILL_NODE_ABORTED;
		}

		/*	invalid response */
		else
		{
			/*	append to the command description	*/
			strFormat.Format("Failed to stop node\r\nResponse: %#04x", ucResponse);
			strCmd_info[iPos] = strFormat;
			return ERROR_INVALID_RESPONSE;
		}
	}

	return 0;
}

/***********************
*	nd_kill_node_abort
*	nd_kill_node_abort sneds a byte to the basestation so the basestation will stop trtying to kill the node. 
*	Command:
*		Length:		1 Byte
*		Byte 1:		0x00		
*   
*   Note:   nd_kill_node_status should be called after aborting to get the status
*
*	PRE:	hPort is a legitimate, open, port handle of type HANDLE
*			nd_kill_node_init was previously called
*
***********************/
int nd_kill_node_abort(/* in */ HANDLE hPort)
{
	
	DWORD	dwBytes_written = 0,	//	number of bytes written to the port
			dwBytes_read = 0;		//	number of bytes read from the port
	int iResult,					//	return value of port read/write functions
		iPos;						//	position in the console that the command was written to
	CString strFormat;				//	formatted string for output to command information window

	unsigned char byteCommand = 0x0;

	/*	send a byte to the base station to stop the kill command */
	if ((iResult = port_write(hPort, &byteCommand, 1, dwBytes_written, iPos)) != 0)
		return iResult;

	strCmd_info[iPos] = "sent byte to abort \"Kill Node\" command";

	return iResult;

}
