// Copyright (C) 2012 MicroStrain, Inc.

/*************************************
*	serial.cpp
*	serial.cpp is the implementation file to the serial
*	port functions.  This file contains all the information for
*	opening, closing, reading, writing, and purging the
*	communications ports.  Please, see each function as well
*	as the Microsoft MSDN documentation for extended information
*	on function calls.
*************************************/

#include "stdafx.h"
#include "2400_sdk.h"

#include "serial.h"

#include "console_commands.h"



/*	strCmd_info is a hash that holds all the strings associated with each command.  When the user
*	selects a command in the console, the index of that selected item is used and is referenced in
*	strCmd_info to show the string that explains the command in the command information area of the
*	demo applications.	*/
extern CMapPosToCString strCmd_info;

/*	crCmd_color is the list of colors for outputting to the console window.  When information is
*	output to the screen, the drawing function looks up the corresponding color based on its
*	position in the console window and sets the text color to that index's color.	*/
extern CMapPosToColor crCmd_color;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/*********************
*	port_open
*	port_open opens the serial port given by lpszPort.  The port is opened
*	with the following specifications:
*	Baud: 921600 or 115200
*	Byte Size: 8
*	Parity:	None
*	Stop Bits:	1
*	Read Timeout Between Characters: 100 ms
*	Input Buffer Size: 2048 bytes
*	Output Buffer Size: 2048 bytes
*	I/O; Synchronous
*	NOTE: The format of the incoming string must be that of the following:
*				\\.\COMX where X is some number
*	The standard COMX format will work only for ports 9 and below.  This issue
*	is documented in the the Microsoft MSDN Knowledge Base article Q115831 available
*	at http://support.microsoft.com/default.aspx?scid=kb;en-us;115831
*	PRE:	lpszPort is an LPCSTR type with the format listed in the note above
*			iBaudRate is the baud rate in which to open the port
*	POST:	if successful, the handle to the port signified by lpszPort has been returned
*			if failed, NULL is returned
*********************/
HANDLE port_open(/* in */ LPCSTR lpszPort, int iBaudRate)
{
	DCB PortDCB;					// Port Control Settings Structure
	COMMTIMEOUTS CommTimeouts;	// Comm port timeout structure

	// Attempt to open the serial port
	HANDLE hPort = CreateFile (	lpszPort,						// Pointer to the name of the port
								GENERIC_READ | GENERIC_WRITE,	// Access (read-write) mode
								0,								// Share mode
								NULL,							// Pointer to the security attribute
								OPEN_EXISTING,					// How to open the serial port
								NULL,							// Port attributes
								NULL);							// Handle to port with attribute
																// to copy

	// If it fails to open the port, return NULL
	if ( hPort == INVALID_HANDLE_VALUE ) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Port failed to open because an invalid handle was returned.";
		crCmd_color[iPos] = COLORREF(COLOR_NEUTRAL);
		return NULL;
	}

	PortDCB.DCBlength = sizeof (DCB);		//	set size bounds for the DCB object   

	// Get the default port setting information.
	if (!GetCommState (hPort, &PortDCB)) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not obtain comm port settings.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

  	PortDCB.BaudRate = iBaudRate;		//	communicate at baud rate specified by user
	PortDCB.ByteSize = 8;				//	byte size of 8
	PortDCB.Parity = NOPARITY;			//	no parity
	PortDCB.StopBits = ONESTOPBIT;		//	one stop bit
	
	//	Configure the port with the settings we just adjusted
	if (!SetCommState (hPort, &PortDCB)) {
		/*	possible close here	*/
		return NULL;
	}

	/*	Set the input and output buffers of the port to 2 MB.  These values
	*	are not very aggressive and make sure that there will almost always
	*	be room in the port for incoming data.  This number is intended for
	*	machines that may not be quite as fast as they should be during a stream.
	*	If you have newer hardware (even 600 Mhz and above) this number can be lowered
	*	greatly.	*/
	if (!SetupComm(hPort, 2048, 2048)) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not set comm port settings.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

	/*	Get the timeout information for the port.	*/
	if (!GetCommTimeouts(hPort, &CommTimeouts)) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not obtain comm port timeouts.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

	/*	Adjust the port timeouts.  For information on this structure, please, search Microsoft's MSDN documentation
	*	for COMMTIMEOUTS or reference their website directly at http://msdn.microsoft.com/library/default.asp?url=/library/en-us/devio/base/commtimeouts_str.asp
	*/
	CommTimeouts.ReadIntervalTimeout = 500;			//	read interval timeout
	CommTimeouts.ReadTotalTimeoutMultiplier = 100;	//	read timeout multiplier 
	CommTimeouts.ReadTotalTimeoutConstant = 1000;	//	read timeout constant
	CommTimeouts.WriteTotalTimeoutMultiplier = 0;	//	write timeout multiplier 
	CommTimeouts.WriteTotalTimeoutConstant = 0;		//	write timeout constant 

	// Set the time-out parameters for all read and write operations on the port. 
	if (!SetCommTimeouts(hPort, &CommTimeouts)) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not set comm port timeouts.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

	/*	Get comm mask	*/
	DWORD dwEventMask;
	/*if (!GetCommMask(hPort, &dwEventMask)) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not set comm port timeouts.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}*/
	dwEventMask = EV_RXCHAR;
	if (!SetCommMask(hPort, dwEventMask)) {
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not set comm port timeouts.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

	port_purge(hPort);	//	purge any left over data out of the port

	int iPos = console_write("Port opened successfully");
	CString portOpenedString;
	portOpenedString.Format("Port opened at %i baud, byte size of 8, no parity, and one stop bit", iBaudRate);
	strCmd_info[iPos] = "";
	crCmd_color[iPos] = COLOR_NEUTRAL;

	return hPort;
}

/*********************
*	port_close
*	port_close takes in the handle of a port signified by hPort
*	and attempts to close that port.
*	PRE:	hPort is a legitimate port HANDLE
*	POST:	if successful, 0 has been returned
*			if failed, -1 has been returned
*********************/
int port_close(/* in */ HANDLE hPort)
{
	if (!CloseHandle (hPort)) {
		int iPos = console_write("Port close failure");
		strCmd_info[iPos] = "Port failed to close";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return -1;
	}

	int iPos = console_write("Port closed successfully");
	strCmd_info[iPos] = "Port closed";
	crCmd_color[iPos] = COLOR_NEUTRAL;
	return 0;
}

/*********************
*	port_purge
*	port_purge takes in the handle of a port signified by hPort and
*	purges the input and output buffers if they exist.
*	PRE:	hPort is a legitimate port HANDLE
*	POST:	if successful, 0 has been returned
*			if failed, -1 has been returned
*********************/
int port_purge(/* in */ HANDLE hPort)
{
	/*	Purge the input and output buffers.  For more information on the PurgeComm function, search Microsoft's
	*	MSDN documentation for PurgeComm or visit the function directly online at
	*	http://msdn.microsoft.com/library/default.asp?url=/library/en-us/devio/base/purgecomm.asp	
	*	We only need to purge the receive and transfer buffers and not terminate any waiting transmissions
	*	because the port_open function opens the port in a synchronous reading mode.	*/
	if (!PurgeComm(hPort, PURGE_RXCLEAR | PURGE_TXCLEAR)) {	// clear the input buffer
		return -1;
	}
	return 0;
}

/*********************
*	port_write
*	port_write takes in a the handle of a port signified by hPort and attempts
*	to write data to that port.
*	PRE:	hPort is a legitimate port HANDLE
*			ucCmd is a pointer to an unsigned char array.  This array is not NULL terminated
*			dwLength is the length of ucCMD of type DWORD
*			dwBytes_written will hold the number of bytes written to the serial port after WriteFile
*				has been called.
*			iPos is a legitimate integer
*	POST:	iPos holds the line position that any data was written to in the console.
*			if successful, 0 has been returned and dwBytes_written holds the number of bytes written
*				to the serial port.
*			if failed, -1 has been returned and dwBytes_written holds the number of bytes written 
*				to the serial port.
*********************/
int port_write(	/* in */ HANDLE hPort, /* in */ unsigned char *ucCmd, 
				/* in */ DWORD dwLength, /* out */ DWORD & dwBytes_written,
				/* out */ int & iPos)
{

	iPos = console_write_command(ucCmd, dwLength);
	crCmd_color[iPos] = COLOR_SEND;

	port_purge(hPort);

	/*	Write the data to the port given by the hPort handle.	*/
	if (!WriteFile(hPort, ucCmd, dwLength, &dwBytes_written, NULL)) {
		return -1;
	}
	/*	Number of bytes to be written != number of bytes written	*/
	if (dwLength != dwBytes_written)
		return ERROR_MISMATCHING_BYTE_LENGTH;

	return 0;
}

/*********************
*	port_read
*	port_read takes in the handle of the port signified by hPort and attempts to read
*	data from that port.
*	PRE:	hPort is a legitimate port HANDLE
*			ucResult is a poitner to an unsigned char array.  This array is written to, but
*				memory must be allocated by the user before the function call.
*			dwLength holds the number of bytes to read from the serial port.  This value should
*				be the length of ucResult.
*			dwBytes_written will hold the number of bytes read from the serial port after
*				ReadFile is called
*			iPos is a legitimate integer
*	POST:	iPos holds the line position that any data was written to in the console.
*			if successful, 0 has been returned and dwBytes_read holds the number of bytes read
*				from the serial port.
*			if failed, -1 has been returned and dwBytes_read holds the number of bytes read 
*				from the serial port.
*********************/
int port_read(	/* in */ HANDLE hPort, /* in/out */ unsigned char *ucResult,
				/* in */ DWORD dwLength, /* out */ DWORD & dwBytes_read,
				/* out */ int & iPos)
{
	static int iPrev_pos = 0;	//	keeps track of the previous position for multiple line, color support
	/*	with the read function, a false return is a real problem.  a good response requires
	*	verification that length == result_len	*/
	if (!ReadFile(hPort, ucResult, dwLength, &dwBytes_read, NULL)) {
		iPos = console_write_command(ucResult, dwBytes_read);	//	output our bad result
		return -1;
	}
	/*	output the result we got, even if it's the wrong result	*/
	int iSize = static_cast<int>(crCmd_color.GetSize());
	iPos = console_write_command(ucResult, dwBytes_read);
	do {
		crCmd_color[iSize] = COLOR_RECIEVE;
	} while (iSize++ < iPos);
	//iPrev_pos = iPos;

	if (dwLength != dwBytes_read)
		return ERROR_MISMATCHING_BYTE_LENGTH;

	return 0;
}

int port_update_read_timeout( /* in */ HANDLE hPort, /* in */ int timeout /*= 500*/)
{
	COMMTIMEOUTS CommTimeouts;

	/*	Get the timeout information for the port.	*/
	if (!GetCommTimeouts(hPort, &CommTimeouts)) 
	{
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not obtain comm port timeouts.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

	/*	Adjust the port timeouts.  For information on this structure, please, search Microsoft's MSDN documentation
	*	for COMMTIMEOUTS or reference their website directly at http://msdn.microsoft.com/library/default.asp?url=/library/en-us/devio/base/commtimeouts_str.asp
	*/
	CommTimeouts.ReadIntervalTimeout = timeout;			//	allow up to 100 ms to elapse between characters  

	// Set the time-out parameters for all read and write operations on the port. 
	if (!SetCommTimeouts(hPort, &CommTimeouts)) 
	{
		int iPos = console_write("Port failed to open");
		strCmd_info[iPos] = "Could not set comm port timeouts.";
		crCmd_color[iPos] = COLOR_NEUTRAL;
		return NULL;
	}

	return 0;
}